'''
Created on Nov 3, 2012

@author: vcarlos
'''
import abc


#===============================================================================
# MemorySystem
#===============================================================================
class MemorySystem(object):
    '''
    classdocs
    '''

    def __init__(self):
        '''
        Constructor
        '''
        self.total_energy = 0

        
    @abc.abstractmethod  
    def Read(self, address):
        return
    
    
#===============================================================================
# FlashMemorySystem
#===============================================================================
class FlashMemorySystem(MemorySystem):
    
    def __init__(self, flash_memory):
        super(FlashMemorySystem, self).__init__()
        self.flash = flash_memory
        
        self.flash_energy = 0
        
        
    def Read(self, address):
        
        self.flash_energy += self.flash.energy_per_read
        return 1
        
        
    def __str__(self, *args, **kwargs):
        result = "\nMemory System composed of"
        result += "\n"
        result += "\nFlash Memory"
        result += "\n"
        result += "Total Energy: " + str(self.flash_energy)
        
        return result         
    
    
#===============================================================================
# CachedMemorySystem    
#===============================================================================
class CachedMemorySystem(MemorySystem):
    
    
    def __init__(self, dm_cache, flash_memory):
        super(CachedMemorySystem, self).__init__()
        self.dm_cache = dm_cache
        self.flash = flash_memory
        self.miss_count = 0
        self.hit_count = 0
        
        self.cache_energy = 0
        self.flash_energy = 0
        
    
    def Read(self, address):
        
        hit = self.dm_cache.Read(address)
        self.cache_energy += self.dm_cache.energy_per_read
        if hit == 1:
            self.hit_count += 1
            return 1
        else:
            self.miss_count += 1
            self.cache_energy += self.dm_cache.energy_per_write
            self.flash_energy += self.flash.energy_per_read
            return 0
        
            
    def __str__(self, *args, **kwargs):
        result = ""
#        result = str(self.dm_cache.number_words) + ";"
        result +=  str(self.dm_cache.size)  + ";"
        result +=  str(self.miss_count)  + ";"
        result +=  str(self.hit_count)  + ";"
        result +=  str(self.cache_energy)  + ";"
        result +=  str(self.flash_energy)  + ";"
        result +=  str((self.flash_energy + self.cache_energy)/1000.) + "\n"
        
#        result += "\n\n number of sets: " + str(len(self.dm_cache.data))
        
        return result
#            
#    def __str__(self, *args, **kwargs):
#        result = "\nMemory System composed of"
#        result += "\nCache:"
#        result += "\n\tDM: " + str(self.dm_cache.size) + " bytes"
#        result += "\n\t\tMapping: " + self.dm_cache.mapping
#        result += "\n\t\tTotal Misses: " + str(self.miss_count)
#        result += "\n\t\tTotal Hits: " + str(self.hit_count)
#        result += "\n\t\tConflict Misses: " + str(self.dm_cache.GetTotalMissConflicts())
#        result += "\n\t\tEnergy: " + str(self.cache_energy)
#        result += "\n"
#        result += "\nFlash Memory"
#        result += "\n\tEnergy: " + str(self.flash_energy)
#        result += "\n"
#        result += "Total Energy: " + str(self.flash_energy + self.cache_energy)
#        
##        result += "\n\n number of sets: " + str(len(self.dm_cache.data))
#        
#        return result
            
    
#===============================================================================
# PhasedDMCacheMemorySystem
#===============================================================================
class PhasedDMCacheMemorySystem(MemorySystem):
    
    def __init__(self, phased_cache, flash_memory):
        super(PhasedDMCacheMemorySystem, self).__init__()
        self.phased_cache = phased_cache
        self.flash = flash_memory
        self.miss_count = 0
        self.hit_count = 0
        
        self.cache_energy = 0
        self.flash_energy = 0
        self.data_array_energy = 0
        self.tag_array_energy = 0
        
    
    def Read(self, address):
        
        hit = self.phased_cache.Read(address)
        self.cache_energy += self.phased_cache.GetEnergyPerTagRead()
        self.tag_array_energy += self.phased_cache.GetEnergyPerTagRead()
        if hit == 1:
            self.cache_energy += self.phased_cache.GetEnergyPerDataRead()
            self.data_array_energy += self.phased_cache.GetEnergyPerDataRead()
            self.hit_count += 1
            return 1
        else:
            self.miss_count += 1
            self.cache_energy += self.phased_cache.GetEnergyPerWrite()
            self.data_array_energy += self.phased_cache.energy_per_write_data
            self.tag_array_energy += self.phased_cache.energy_per_write_tag
            self.flash_energy += self.flash.energy_per_read
            return 0
        
            
    def __str__(self, *args, **kwargs):
        result = ""
#        result = str(self.phased_cache.number_words) + ";"
        result +=  str(self.phased_cache.size)  + ";"
        result +=  str(self.miss_count)  + ";"
        result +=  str(self.hit_count)  + ";"
        result +=  str(self.cache_energy)  + ";"
        result +=   str(self.tag_array_energy)  + ";"
        result +=   str(self.data_array_energy)  + ";"
        result +=  str(self.flash_energy)  + ";"
        result +=  str((self.flash_energy + 2*self.cache_energy)/1000.) + "\n"            
            
        return result
                
#    def __str__(self, *args, **kwargs):
#        result = "\nMemory System composed of"
#        result += "\nCache:"
#        result += "\n\t\tMapping: " + self.phased_cache.mapping
#        result += "\n\t\tTotal Misses: " + str(self.miss_count)
#        result += "\n\t\tTotal Hits: " + str(self.hit_count)
#        result += "\n\t\tEnergy: " + str(self.cache_energy)
#        result += "\n"
#        result += "\nFlash Memory"
#        result += "\n\tEnergy: " + str(self.flash_energy)
#        result += "\n"
#        result += "Total Energy: " + str(self.flash_energy + self.cache_energy)
#        
##        result += "\n\n number of sets: " + str(len(self.phased_cache.data))
#        
#        return result            
    
#===============================================================================
# PhasedCacheMemorySystem    
#===============================================================================
class PhasedCacheMemorySystem(PhasedDMCacheMemorySystem):
    
    pass
#    def __init__(self, phased_cache, flash_memory):
#        super(PhasedCacheMemorySystem, self).__init__()
#        self.phased_cache = phased_cache
#        self.flash = flash_memory
#        self.miss_count = 0
#        self.hit_count = 0
#        
#        self.cache_energy = 0
#        self.flash_energy = 0
#        
#    
#    def Read(self, address):
#        
#        hit = self.phased_cache.Read(address)
#        self.cache_energy += self.phased_cache.GetEnergyPerTagRead()
#        if hit == 1:
#            self.cache_energy += self.phased_cache.GetEnergyPerDataRead()
#            self.hit_count += 1
#            return 1
#        else:
#            self.miss_count += 1
#            self.cache_energy += self.phased_cache.GetEnergyPerWrite()
#            self.flash_energy += self.flash.energy_per_read
#            return 0
        
            
#    def __str__(self, *args, **kwargs):
#        result = "\nMemory System composed of"
#        result += "\nCache:"
#        result += "\n\t\tMapping: " + self.phased_cache.mapping
#        result += "\n\tNWay: " + str(self.phased_cache.size) + " bytes"
#        result += "\n\t\t# of Ways: " + str(self.phased_cache.number_of_ways)
#        result += "\n\t\tTotal Misses: " + str(self.miss_count)
#        result += "\n\t\tTotal Hits: " + str(self.hit_count)
#        result += "\n\t\tEnergy: " + str(self.cache_energy)
#        result += "\n"
#        result += "\nFlash Memory"
#        result += "\n\tEnergy: " + str(self.flash_energy)
#        result += "\n"
#        result += "Total Energy: " + str(self.flash_energy + self.cache_energy)
#        
##        result += "\n\n number of sets: " + str(len(self.phased_cache.data))
#        
#        return result



#===============================================================================
# WayHaltingCacheMemorySystem
#===============================================================================
class WayHaltingCacheMemorySystem(MemorySystem):
            
    def __init__(self, way_halting_cache, flash_memory):
        super(WayHaltingCacheMemorySystem, self).__init__()
        self.way_halting_cache = way_halting_cache
        self.flash = flash_memory
        self.miss_count = 0
        self.hit_count = 0
        
        self.cache_energy = 0
        self.flash_energy = 0
        
    
    def Read(self, address):
#        print "-"*30 + str(address) + "-"*30 
        hit = self.way_halting_cache.Read(address)
#        print self.way_halting_cache
#        print "Energy read: ", self.way_halting_cache.GetEnergyPerRead()
#        print "hit ", hit
        self.cache_energy += self.way_halting_cache.GetEnergyPerRead()
#        import pdb;pdb.set_trace()
        if hit == 1:
            self.hit_count += 1
            return 1
        else:
            self.miss_count += 1
            self.cache_energy += self.way_halting_cache.GetEnergyPerWrite()
            self.flash_energy += self.flash.energy_per_read
            return 0
        
        
#    def __str__(self, *args, **kwargs):
#        result = ""
##        result = str(self.way_halting_cache.number_words) + ";"
#        result +=  str(self.way_halting_cache.size)  + ";"
##        result +=  str(self.way_halting_cache.number_of_ways) + ";"
##        result += str(self.way_halting_cache.total_number_of_haltings) + ";"
##        result += str(1.0 * self.way_halting_cache.total_ways / self.way_halting_cache.number_of_accesses) + ";"    
#        result +=  str(self.miss_count)  + ";"
#        result +=  str(self.hit_count)  + ";"
#        result +=  str(self.cache_energy)  + ";"
#        result +=  str(self.flash_energy)  + ";"
#        result +=  str((self.flash_energy + self.cache_energy)/1000.) + "\n"            
#            
#        return result        
            
    def __str__(self, *args, **kwargs):
        result = "\nMemory System composed of"
        result += "\nCache:"
        result += "\n\tNWay: " + str(self.way_halting_cache.size) + " bytes"
        result += "\n\t\t# of Ways: " + str(self.way_halting_cache.number_of_ways)
        result += "\n\t\t# of Haltings: " + str(self.way_halting_cache.total_number_of_haltings)
        result += "\n\t\tAvg# of Ways: " + str(1.0 * self.way_halting_cache.total_ways / self.way_halting_cache.number_of_accesses)
        result += "\n\t\tTotal Misses: " + str(self.miss_count)
        result += "\n\t\tTotal Hits: " + str(self.hit_count)
        result += "\n\t\tEnergy: " + str(self.cache_energy)
        result += "\n"
        result += "\nFlash Memory"
        result += "\n\tEnergy: " + str(self.flash_energy)
        result += "\n"
        result += "Total Energy: " + str(self.flash_energy + self.cache_energy)
        
#        result += "\n\n number of sets: " + str(len(self.way_halting_cache.data))
#        for w in self.way_halting_cache.data:
#            print "way---"
#            halt = w[0]
#            for k, v in halt.iteritems():
#                print '{0:08b}'.format(k), '{0:018b}'.format(v)  
#                print k, v
#            print w
        
        return result

            
            
#===============================================================================
# SPMMemorySystem
#===============================================================================
class SPMMemorySystem(MemorySystem):
    
    def __init__(self, spm, flash_memory):
        super(SPMMemorySystem, self).__init__()
        self.spm = spm
        self.flash = flash_memory
        
        self.spm_energy = 0
        self.flash_energy = 0
        self.cfa = 0
        self.hit_count = 0
        
    
    def Read(self, address):
        
        hit = self.spm.Read(address)
        self.hit_count += hit
        if hit:
            self.spm_energy += self.spm.energy_per_read
            return 1       
        else:
#            print "address going to flash: ", hex(address)
            self.cfa += 1
            self.flash_energy += self.flash.energy_per_read
            return 0
            
            
    def __str__(self, *args, **kwargs):
        result = "\nMemory System composed of"
        result += "\nSPM:"  + str(self.spm.size) + " bytes"
        result += "\n\tEnergy: " + str(self.spm_energy)
        result += "\n"
        result += "\n\tHit Count: " + str(self.hit_count)
        result += "\n"        
        result += "\nFlash Memory"
        result += "\n\tEnergy: " + str(self.flash_energy)
        result += "\n"
        result += "\nTotal Energy: " + str(self.flash_energy + self.spm_energy)
        
#        result += "\nFunctions mapped:"
##        for f in self.spm.mapping:
##            result += "\n" + str(f)
#        for f in self.spm.status.keys():
#            result += "\n" + f + " : "  + str(self.spm.status[f])
        
        return result            
            


#===============================================================================
# CachedWithFunctionsMemorySystem
#===============================================================================
class CachedWithFunctionsMemorySystem(CachedMemorySystem):
    
    def __init__(self, dm_cache, flash_memory, list_of_functions):
        super(CachedWithFunctionsMemorySystem, self).__init__(dm_cache, flash_memory)
        
        self.mapping = list_of_functions
        self.total = 0
    
    
    def Read(self, address):
        '''
            "Read" from memory and return 1 if found, 0 otherwise.
        '''
        hit = super(CachedWithFunctionsMemorySystem, self).Read(address)
        self.total+=1
        
        for function in self.mapping:
            if address >= function.starting_address and address <= function.final_address:
                if hit == 1:
                    function.hit_count += 1
                else:
                    function.miss_count += 1
                break
                
        return hit
    
    
#===============================================================================
# SPMCacheMemorySystem    
#===============================================================================
class SPMCacheMemorySystem(MemorySystem):
    
    def __init__(self, dm_cache, spm, flash_memory):
        super(SPMCacheMemorySystem, self).__init__()
        
        self.cached_ms = CachedMemorySystem(dm_cache, flash_memory)
        self.spm_ms = SPMMemorySystem(spm, flash_memory)
        
        
    def Read(self, address):
        
        hit = self.spm_ms.Read(address)
        if not hit:
            hit = self.cached_ms.Read(address)
            
        return hit
            
    def __str__(self, *args, **kwargs):
        result = "\nMemory System composed of"
        result += "\nCache:"
        result += "\n\tDM: " + str(self.cached_ms.dm_cache.size) + " bytes"
        result += "\n\t\tMapping: " + self.cached_ms.dm_cache.mapping
        result += "\n\t\tTotal Misses: " + str(self.cached_ms.miss_count)
        result += "\n\t\tTotal Hits: " + str(self.cached_ms.hit_count)
        result += "\n\t\tConflict Misses: " + str(self.cached_ms.dm_cache.GetTotalMissConflicts())
        result += "\n\t\tEnergy: " + str(self.cached_ms.cache_energy)
        result += "\n"        
        result += "\n\tSPM:"  + str(self.spm_ms.spm.size) + " bytes"
        result += "\n\t\tEnergy: " + str(self.spm_ms.spm_energy)
        result += "\n"
        result += "\nFlash Memory"
        result += "\n\tEnergy: " + str(self.cached_ms.flash_energy)
        result += "\n"
        result += "\nTotal Energy: " + str(self.cached_ms.flash_energy + self.spm_ms.spm_energy + self.cached_ms.cache_energy)
        
        return result        

    