'''
Created on Nov 3, 2012

@author: vcarlos
'''
from mem.memory import Memory

class SPM(Memory):
    '''
    In this abstraction of scratchpad, we will have mapping of all functions that are
    located in the scratchpad. So, instead of a contiguous address space, lets say, from 0x0000 to 0xFFFF,
    we will actually just have a mapping of FunctionN (address of instruction calling the function) to: yes, 
    it is on the scratchpad, or no, it is not in the scratchpad.
    
    '''


    def __init__(self, size, energy_per_read):
        '''
        Constructor
        '''
        super(SPM, self).__init__(size, energy_per_read)
        # contain the function objects that are mapped to scratchpad
        self.mapping = []
        
        # auxiliary data structure to return the actual number of "hits" for each function
        # to be compared with the data on analysis.txt
        self.status = {}
        
        self.size = size
        
        
        
    def Read(self, address):
        '''
            "Read" from memory and return 1 if found, 0 otherwise.
        '''
        for function in self.mapping:
            if address >= function.starting_address and address <= function.final_address:
                if self.status.has_key(function.name):
                    count = self.status[function.name]
                    self.status[function.name] = count + 1
                else:
                    self.status[function.name] = 1
                
                return 1
        return 0        
