'''
Created on Nov 14, 2012

@author: vcarlos
'''
from ctypes import *
import xlrd
from spm import SPM
from mem.memory_system import CachedWithFunctionsMemorySystem


class Function(object):
    
    def __init__(self, times_function_called, number_fetches, function_name, starting_address, size):
        self.times_function_called = times_function_called
        self.number_fetches = number_fetches
        self.name = function_name
        self.starting_address = starting_address
        self.size = size
        final_address = starting_address + size  
        self.final_address = final_address
        # see later where to put them
        self.hit_count = 0
        self.miss_count = 0
        
        
    def __str__(self, *args, **kwargs):
        res = "---------------Function------------------"
        res += "\nName: " + self.name
        res += "\nNumber of Fetches: " + str(self.number_fetches)
        res += "\nstarting_address: " + str(self.starting_address)
        res += "\nfinal_address: " + str(self.final_address)
        res += "\nsize: " + str(self.size)
        res += "\nhit_count: " + str(self.hit_count)
        res += "\nmiss_count: " + str(self.miss_count)
        res += "\n"
        
        return res
        

def CreateListOfFunction(excel_worksheet):
    
    book = xlrd.open_workbook(excel_worksheet)
    sheet = book.sheet_by_index(0)
    
    list_of_functions = []
    
    for row in xrange(sheet.nrows-1):
        row += 1
        times_function_called = int(sheet.cell_value(row, 0))
        number_fetches = int(sheet.cell_value(row, 1))
        function_name = sheet.cell_value(row, 2)
        starting_address = int(sheet.cell_value(row, 3), 16) - 1
        size = int(sheet.cell_value(row, 4))
        f = Function(times_function_called, number_fetches, function_name, starting_address, size)
        list_of_functions.append(f)
        
    return list_of_functions


def LoadMinknapSharedLibrary():
    cdll.LoadLibrary("/home/vcarlos/workspace/test/src/minknap.so")
    libc = CDLL("/home/vcarlos/workspace/test/src/minknap.so")
    return libc



def RunMinknap(list_of_functions, flash_energy_per_read, sram_energy_per_read, spm_size):
    libc = LoadMinknapSharedLibrary()
    number_of_functions = len(list_of_functions)
    
    # create the input/output arrays for the minknap function
    n = number_of_functions
    # x has the solution array (0 or 1)
    x = (c_uint*n)()
    # p has the profit function. #fetches * (flash - sram)
    p = (c_uint*n)()
    # w is the cost, in number of bytes
    w = (c_uint*n)()
    c = spm_size
    
    base_profit = flash_energy_per_read - sram_energy_per_read
    
    # fill in the arrays with data from functions objects
    for i, function in enumerate(list_of_functions):
#        print "int(round(function.number_fetches * base_profit))", int(round(function.number_fetches * base_profit))
        p[i] = int(round(function.number_fetches * base_profit))
        w[i] = function.size
        
    res = libc.minknap(n, p, w, x, c)
    
    return res, x


def SPMBuilder(excel_worksheet, flash_energy_per_read, sram_energy_per_read, spm_size):
    '''
    Constructs a SPM based on the trace analysis excel sheet.
    
    It makes used of the shared library minknap.so to determine which functions should be placed on
    the SPM.
    
    The end results is an SPM object, with the functions that are allocated to it. 
    
    A function is an object of class Function.
    '''

    # first create list of functions based on the excel work sheet
    list_of_functions = CreateListOfFunction(excel_worksheet)
    res, solution = RunMinknap(list_of_functions, flash_energy_per_read, sram_energy_per_read, spm_size)
    
    #now I can create the SPM
    spm = SPM(spm_size, sram_energy_per_read)
    for i, sol in enumerate(solution):
        if sol == 1:
            func = list_of_functions[i]
#            print func
            spm.mapping.append(func)
            
    return spm


def RunMinknap2(list_of_functions, flash_energy_per_read, sram_energy_per_read, spm_size, cache_energy_per_read, cache_energy_per_write):
    libc = LoadMinknapSharedLibrary()
    number_of_functions = len(list_of_functions)
    
    # create the input/output arrays for the minknap function
    n = number_of_functions
    # x has the solution array (0 or 1)
    x = (c_uint*n)()
    # p has the profit function. 
    p = (c_uint*n)()
    # w is the cost, in number of bytes
    w = (c_uint*n)()
    c = spm_size

    # here, spm.mapping has all functions
    # now, I need to run a system with a cache, having the knowledge of those functions to determine
    # the number of misses that happened on each function, so I can come with the correct profit value
    # for putting a particular function into the scratchpad
    # remember that, without a cache, the profit is all fetches * main_memory_energy - all_fetches * spm_energy
    # now, because there is a cache, not all_fetches go to MM, some of them go to cache, so the calculation is
    # number_hits * cache_energy + number_miss * (cache_energy*2 + mm_energy) - all_fetches * spm_energy    
    
    hit_energy = cache_energy_per_read
    miss_energy = cache_energy_per_read + cache_energy_per_write + flash_energy_per_read
    
    # fill in the arrays with data from functions objects
    for i, function in enumerate(list_of_functions):
        energy_mm  = function.hit_count * hit_energy + function.miss_count * miss_energy
        energy_spm = function.number_fetches * sram_energy_per_read 
#        print "int(round(energy_mm - energy_spm))", int(round(energy_mm - energy_spm))
        p[i] = int(round(energy_mm - energy_spm)) 
        w[i] = function.size
        
    res = libc.minknap(n, p, w, x, c)
    
    return res, x


def SPMBuilderWithCache(excel_worksheet, flash_energy_per_read, sram_energy_per_read, spm_size, simulation):
    '''
    Constructs a SPM based on the trace analysis excel sheet.
    
    It makes used of the shared library minknap.so to determine which functions should be placed on
    the SPM.
    
    The end results is an SPM object, with the functions that are allocated to it. 
    
    A function is an object of class Function.
    '''

    # first create list of functions based on the excel worksheet
    list_of_functions = CreateListOfFunction(excel_worksheet)
    
    dm_cache = simulation.memory_system.dm_cache
    flash_memory = simulation.memory_system.flash
    simulation.memory_system = CachedWithFunctionsMemorySystem(dm_cache, flash_memory, list_of_functions)
    
    simulation.Run()
    
    cache_energy_per_read = dm_cache.energy_per_read
    cache_energy_per_write = dm_cache.energy_per_write
    
    res, solution = RunMinknap2(list_of_functions, flash_energy_per_read, sram_energy_per_read, spm_size, cache_energy_per_read, cache_energy_per_write)
    
    total = 0
    
    #now I can create the SPM
    spm = SPM(spm_size, sram_energy_per_read)
    for i, sol in enumerate(solution):
        func = list_of_functions[i]
        total += func.hit_count + func.miss_count
        if sol == 1:
#            print func
            spm.mapping.append(func)
            
    print "Total", total
    assert total == 381661            
    return spm
    
    
#def Run():
#    from scenarios import FactoryDMCache128Words
#    sim = FactoryDMCache128Words.Build()    
#    SPMBuilderWithCache("/home/vcarlos/workspace/simulation/input/list_of_functions.xls", 0.1593, 0.0593, 4096, sim)
#
#
#Run()
    
    
#===============================================================================
# main
#===============================================================================
if __name__ == '__main__':
    pass    