'''
Created on Nov 3, 2012

@author: vcarlos
'''
from mem.cache import DMCache, XORDMCache, PhasedNWayCache, PhasedDMCache, \
    WayHaltingCache
from mem.flash import FlashMemory
from mem.memory_system import FlashMemorySystem, SPMMemorySystem, \
    CachedMemorySystem, SPMCacheMemorySystem, PhasedCacheMemorySystem, \
    PhasedDMCacheMemorySystem, WayHaltingCacheMemorySystem
from mem.spm_builder import SPMBuilderWithCache, SPMBuilder
from param import sim1, sim2, sim3
from sim import Simulation
import helper
from helper import GetNumberOfWords

#===============================================================================
# General Simulation Parameters
#===============================================================================
# ALL ENERGY VALUES IN nJ
MAIN_MEMORY_SIZE = 262144  # bytes
FLASH_ENERGY_PER_READ = 0.500  # nJ
SPM_ENERGY_PER_READ_512 = 0.0186 
SPM_ENERGY_PER_READ_1K = 0.0222
SPM_ENERGY_PER_READ_2K = 0.024
SPM_ENERGY_PER_READ_4K = 0.0312
        
#===============================================================================
# Current Simulation Parameters
#===============================================================================
current_sim = sim1
trace_file = current_sim.trace_file
analysis_file = current_sim.analysis_file

EER = helper.ExcelEnergyReader(current_sim.memory_file)
GEV = EER.GetEnergyValue


# LIST_OF_FUNCTIONS_WORK_SHEET = "/home/vcarlos/workspace/simulation/input/list_of_functions.xls"

#===============================================================================
# Cache Factories
#===============================================================================

#===============================================================================
# PhasedCaches
#===============================================================================
class CacheFactory2Way16Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 128  # bytes
        mapping = "000000000000111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory2Way32Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 256  # bytes
        mapping = "000000000001111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory2Way64Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 512  # bytes
        mapping = "000000000011111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory2Way128Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 1024  # bytes
        mapping = "000000000111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory2Way256Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 2048  # bytes
        mapping = "000000001111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory2Way512Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 4096  # bytes
        mapping = "000000011111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory2Way1024Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 8192  # bytes
        mapping = "000000111111111100"
        
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    
    

class CacheFactory2Way2048Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 2
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 16384  # bytes
        mapping = "000001111111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    


class CacheFactory4Way8Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 128  # bytes
        
        mapping = "000000000000011100"
        
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)



class CacheFactory4Way16Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 256  # bytes
        mapping = "000000000000111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory4Way32Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
                
        cache_size = 512  # bytes
        mapping = "000000000001111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory4Way64Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 1024  # bytes
        mapping = "000000000011111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory4Way128Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 2048  # bytes
        mapping = "000000000111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory4Way256Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 4096  # bytes
        mapping = "000000001111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory4Way512Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 8192  # bytes
        mapping = "000000011111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    

class CacheFactory4Way1024Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 16384  # bytes
        mapping = "000000111111111100"
        
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    
    

class CacheFactory4Way2048Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 32668  # bytes
        mapping = "000001111111111100"
        
        return PhasedNWayCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
    
    
#===============================================================================
# Phased Base Lines
#===============================================================================
class CacheFactory32WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
                
        cache_size = 128  # bytes
        mapping = "000000000001111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory64WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
                
        cache_size = 256  # bytes
        mapping = "000000000011111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory128WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 512  # bytes
        mapping = "000000000111111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory256WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 1024  # bytes
        mapping = "000000001111111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory512WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
                
        cache_size = 2048  # bytes
        mapping = "000000011111111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory1024WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
                
        cache_size = 4096  # bytes
        mapping = "000000111111111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory2048WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        # TODO: GET NUMBERS
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 176  # bytes
        mapping = "000000000001111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

class CacheFactory4096WordsPhasedCacheBaseline():

    @classmethod
    def Build(cls):
        
        # TODO GET NUMBERS
        energy_per_read_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'r')
        energy_per_read_data = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write_tag = GEV("Tag", GetNumberOfWords(cls.__name__), 'w')
        energy_per_write_data = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 176  # bytes
        mapping = "000000000001111100"
        
        return PhasedDMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read_tag, energy_per_read_data, energy_per_write_tag, energy_per_write_data)
  

#===============================================================================
# Way Halting
#===============================================================================
class CacheFactoryWayHalting4Way8Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 13

        energy_per_read = 0.0138 + 0.0072 # approx
        energy_per_write = 0.0144 + 0.0078
        
        cache_size = 128  # bytes
        mapping = "000000000000011100"
        
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)
        
        
class CacheFactoryWayHalting4Way16Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 12
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        
        cache_size = 256  # bytes
        mapping = "000000000000111100"
        
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


class CacheFactoryWayHalting4Way32Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 11
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 512  # bytes
        mapping = "000000000001111100"
                
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


class CacheFactoryWayHalting4Way64Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 10
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 1024  # bytes
        mapping = "000000000011111100"
                
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


class CacheFactoryWayHalting4Way128Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 9
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 2048  # bytes
        mapping = "000000000111111100"
                
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


class CacheFactoryWayHalting4Way256Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 8
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 4096  # bytes
        mapping = "000000001111111100"
                
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


class CacheFactoryWayHalting4Way512Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 7
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 8192  # bytes
        mapping = "000000011111111100"
                
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


class CacheFactoryWayHalting4Way1024Words():
    
    @classmethod
    def Build(cls):
        
        number_of_ways = 4
        
        energy_per_read_halt = 0
        energy_per_write_halt = 0
        
        number_of_halt_tag_bits = 4
        number_of_tag_bits = 6
        
        energy_per_read = GEV("Data", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Data", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 16384  # bytes
        mapping = "000000111111111100"
                
        return WayHaltingCache(MAIN_MEMORY_SIZE, cache_size, mapping, number_of_ways, 
                               energy_per_read_halt, energy_per_read, energy_per_write_halt, energy_per_write, 
                               number_of_tag_bits, number_of_halt_tag_bits)


#===============================================================================
# Baselines
#===============================================================================
class CacheFactory32WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 176  # bytes
        mapping = "000000000001111100"
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    

class CacheFactory64WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 344  # bytes
        mapping = "000000000011111100"
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)   


class CacheFactory128WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 672  # bytes
        mapping = "000000000111111100"
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)       


class CacheFactory256WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 1312  # bytes
        mapping = "000000001111111100"
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    

class CacheFactory512WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 2560  # bytes
        mapping = "000000011111111100"
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)    


class CacheFactory1024WordsBaseline():
    
    @classmethod
    def Build(cls):
#        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
#        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        print energy_per_read
        print energy_per_write
        cache_size = 4998  # bytes
        mapping = "000000111111111100"
                
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    

class CacheFactory2048WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 9728  # bytes
        mapping = "000001111111111100"
                        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    

class CacheFactory4096WordsBaseline():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 18944  # bytes
        mapping = "000011111111111100"
                        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)

    
#===============================================================================
# Indexing
#===============================================================================
class CacheFactory32Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 176  # bytes
        mapping = current_sim.mapping["CacheFactory32Words"]
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)

    

class CacheFactory64Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 344  # bytes
        mapping = current_sim.mapping["CacheFactory64Words"]
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    


class CacheFactory128Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 672  # bytes
        mapping = current_sim.mapping["CacheFactory128Words"]
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)    
       


class CacheFactory256Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 1312  # bytes
        mapping = current_sim.mapping["CacheFactory256Words"]
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    


class CacheFactory512Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 2560  # bytes
        mapping = current_sim.mapping["CacheFactory512Words"]
        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    


class CacheFactory1024Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 4998  # bytes
        mapping = current_sim.mapping["CacheFactory1024Words"]
                
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    


class CacheFactory2048Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 9728  # bytes
        mapping = current_sim.mapping["CacheFactory2048Words"]
                        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    

class CacheFactory4096Words():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 18944  # bytes
        mapping = current_sim.mapping["CacheFactory4096Words"]
                        
        return DMCache(MAIN_MEMORY_SIZE, cache_size, mapping, energy_per_read, energy_per_write)
    

#===============================================================================
# XORing
#===============================================================================

class CacheFactory256WordsXOR():
    
    @classmethod
    def Build(cls):
        energy_per_read = GEV("Line", GetNumberOfWords(cls.__name__), 'r')
        energy_per_write = GEV("Line", GetNumberOfWords(cls.__name__), 'w')
        cache_size = 1312  # bytes
        
        return XORDMCache(MAIN_MEMORY_SIZE, cache_size, "", energy_per_read, energy_per_write)
    
    
#===============================================================================
# Memory System Factories
#===============================================================================
class FactoryFlashMemoryBaselineMemorySystem():
    
    build = current_sim.run['FactoryFlashMemoryBaselineMemorySystem']

    @staticmethod
    def Build():
        
        trace_file = current_sim.trace_file
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = FlashMemorySystem(flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
#===============================================================================
# SPM 512B Scenarios
#===============================================================================
class FactorySPM512MemorySystem():
    
    build = current_sim.run['FactorySPM512MemorySystem']
    
    @staticmethod     
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        spm = SPMBuilder(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size)
        
        memory_system = SPMMemorySystem(spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim



class FactorySPM512WithDMCache256WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache256WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM512WithDMCache256WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache256WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim            
    
    
    
class FactorySPM512WithDMCache128WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache128WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    

    
class FactorySPM512WithDMCache128WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache128WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    
    
class FactorySPM512WithDMCache32WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache32WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim         
    

    
class FactorySPM512WithDMCache32WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache32WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
        
    
class FactorySPM512WithDMCache64WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache64WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    

    
class FactorySPM512WithDMCache64WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache64WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactorySPM512WithDMCache512WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache512WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactorySPM512WithDMCache512WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache512WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM512WithDMCache1024WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache1024WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    

    
class FactorySPM512WithDMCache1024WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache1024WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM512WithDMCache2048WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache2048WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM512WithDMCache2048WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache2048WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    


    
class FactorySPM512WithDMCache4096WordsMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache4096WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM512WithDMCache4096WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM512WithDMCache4096WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_512
        spm_size = 512  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    


    
#===============================================================================
# SPM 1K Scenarios
#===============================================================================
class FactorySPM1kMemorySystem():
    
    build = current_sim.run['FactorySPM1kMemorySystem']
    
    @staticmethod     
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        spm = SPMBuilder(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size)
        
        memory_system = SPMMemorySystem(spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
class FactorySPM1kWithDMCache256WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache256WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM1kWithDMCache256WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache256WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim            
    
    
    
class FactorySPM1kWithDMCache128WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache128WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    

    
class FactorySPM1kWithDMCache128WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache128WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    
    
class FactorySPM1kWithDMCache32WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache32WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim         
    

    
class FactorySPM1kWithDMCache32WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache32WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
        
    
class FactorySPM1kWithDMCache64WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache64WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    

    
class FactorySPM1kWithDMCache64WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache64WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactorySPM1kWithDMCache512WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache512WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactorySPM1kWithDMCache512WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache512WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM1kWithDMCache1024WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache1024WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    

    
class FactorySPM1kWithDMCache1024WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache1024WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM1kWithDMCache2048WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache2048WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM1kWithDMCache2048WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache2048WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    


    
class FactorySPM1kWithDMCache4096WordsMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache4096WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM1kWithDMCache4096WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM1kWithDMCache4096WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_1K
        spm_size = 1024  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    

#===============================================================================
# SPM 2k Scenarios
#===============================================================================
class FactorySPM2kMemorySystem():
    
    build = current_sim.run['FactorySPM2kMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        spm = SPMBuilder(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size)
        
        memory_system = SPMMemorySystem(spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    

class FactorySPM2kWithDMCache256WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache256WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM2kWithDMCache256WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache256WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim            
    
    
    
class FactorySPM2kWithDMCache128WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache128WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    
    
class FactorySPM2kWithDMCache128WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache128WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    
        
    
class FactorySPM2kWithDMCache32WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache32WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim         
    
    
    
class FactorySPM2kWithDMCache32WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache32WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim             
    
    
    
class FactorySPM2kWithDMCache64WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache64WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
class FactorySPM2kWithDMCache64WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache64WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
        
    
class FactorySPM2kWithDMCache512WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache512WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactorySPM2kWithDMCache512WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache512WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
    
class FactorySPM2kWithDMCache1024WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache1024WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
    
class FactorySPM2kWithDMCache1024WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache1024WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
    
    
    
class FactorySPM2kWithDMCache2048WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache2048WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
    
    
class FactorySPM2kWithDMCache2048WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache2048WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
    
    
    
class FactorySPM2kWithDMCache4096WordsMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache4096WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
    
    
class FactorySPM2kWithDMCache4096WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM2kWithDMCache4096WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_2K
        spm_size = 2048  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    

#===============================================================================
# SPM 4k Scenarios
#===============================================================================
class FactorySPM4kMemorySystem():
    
    build = current_sim.run['FactorySPM4kMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        spm = SPMBuilder(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size)
        
        memory_system = SPMMemorySystem(spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    

    
class FactorySPM4kWithDMCache256WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache256WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM4kWithDMCache256WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache256WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache256WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory256Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim            
    
    
    
class FactorySPM4kWithDMCache128WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache128WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    

    
class FactorySPM4kWithDMCache128WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache128WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache128WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory128WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
    
    
class FactorySPM4kWithDMCache32WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache32WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim       
    
    
class FactorySPM4kWithDMCache32WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache32WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache32WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory32WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
    
    
class FactorySPM4kWithDMCache64WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache64WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
class FactorySPM4kWithDMCache64WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache64WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache64WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory64WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactorySPM4kWithDMCache512WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache512WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache512WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory512WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim    
    
 
class FactorySPM4kWithDMCache1024WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache1024WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM4kWithDMCache1024WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache1024WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache1024WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory1024WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
class FactorySPM4kWithDMCache2048WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache2048WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    
    
class FactorySPM4kWithDMCache2048WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache2048WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache2048WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory2048WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
            
            
    
class FactorySPM4kWithDMCache4096WordsMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache4096WordsMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096Words.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim      
    
    
class FactorySPM4kWithDMCache4096WordsBaselineMemorySystem():
    
    build = current_sim.run['FactorySPM4kWithDMCache4096WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = 4096  # bytes    
        
        # this is needed for pre-computation of best SPM mapping
        sim = FactoryDMCache4096WordsBaselineMemorySystem.Build()    
        spm = SPMBuilderWithCache(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size, sim)  
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        dm_cache = CacheFactory4096WordsBaseline.Build()
        
        memory_system = SPMCacheMemorySystem(dm_cache, spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        return sim          
                        

#===============================================================================
# Control Scenario
#===============================================================================
class FactorySPMFullSizeMemorySystem():
    '''
    This scenario assumes a SPM of the size of the full memory.
    Used to compare with the results from analysis.txt
    '''
    
    build = current_sim.run['FactorySPMFullSizeMemorySystem']
    
    @staticmethod
    def Build():
        

        spm_energy_per_read = SPM_ENERGY_PER_READ_4K
        spm_size = MAIN_MEMORY_SIZE  # bytes           
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        spm = SPMBuilder(analysis_file, FLASH_ENERGY_PER_READ, spm_energy_per_read, spm_size)
        
        memory_system = SPMMemorySystem(spm, flash_memory)
        sim = Simulation(memory_system, trace_file)
        
    #    for f in spm.status:
    #        print spm.status[f], " , ", f
    #    print
    #    for f in spm.mapping:
    #        print f.number_fetches, ",", f.name, ",", hex(f.starting_address + 1)
        return sim
        
        
#===============================================================================
# Cache-only Scenarios    
#===============================================================================

class FactoryDMCache32WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache32WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory32WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim


class FactoryDMCache64WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache64WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory64WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim


class FactoryDMCache128WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache128WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory128WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim


class FactoryDMCache256WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache256WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory256WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim
    

class FactoryDMCache512WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache512WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory512WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim
    

class FactoryDMCache1024WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache1024WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory1024WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim


class FactoryDMCache2048WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache2048WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory2048WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim
    
class FactoryDMCache4096WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryDMCache4096WordsBaselineMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory4096WordsBaseline.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryDMCache32WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache32WordsMemorySystem']

    @staticmethod
    def Build():
        
        
         
        dm_cache = CacheFactory32Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim
    
    

class FactoryDMCache64WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache64WordsMemorySystem']

    @staticmethod
    def Build():
        
        
        
        dm_cache = CacheFactory64Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactoryDMCache128WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache128WordsMemorySystem']

    @staticmethod
    def Build():
        
        
        
        dm_cache = CacheFactory128Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim        
    
    
    
class FactoryDMCache256WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache256WordsMemorySystem']

    @staticmethod
    def Build():
        
        dm_cache = CacheFactory256Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
class FactoryDMCache512WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache512WordsMemorySystem']

    @staticmethod
    def Build():
        
        dm_cache = CacheFactory512Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
    
    
class FactoryDMCache1024WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache1024WordsMemorySystem']

    @staticmethod
    def Build():
        
        dm_cache = CacheFactory1024Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

    
class FactoryDMCache2048WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache2048WordsMemorySystem']

    @staticmethod
    def Build():
        
        dm_cache = CacheFactory2048Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        
    
class FactoryDMCache4096WordsMemorySystem():
    
    build = current_sim.run['FactoryDMCache4096WordsMemorySystem']

    @staticmethod
    def Build():
        
        dm_cache = CacheFactory4096Words.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        
    

class FactoryDMCache256WordsXORMemorySystem():
    
    build = current_sim.run['FactoryDMCache256WordsXORMemorySystem']
    
    @staticmethod
    def Build():
        
        
                
        dm_cache = CacheFactory256WordsXOR.Build()
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = CachedMemorySystem(dm_cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


#===============================================================================
# Phased Cache Scenarios    
#===============================================================================

class FactoryPhasedCache2Way16WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way16WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way16Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way32WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way32WordsMemorySystem']

    @staticmethod
    def Build():
        
        print 'before'
        cache = CacheFactory2Way32Words.Build()
        print 'after'
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way64WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way64WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way64Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way128WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way128WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way128Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way256WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way256WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way256Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way512WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way512WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way512Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way1024WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way1024WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way1024Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache2Way2048WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache2Way2048WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2Way2048Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way8WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way8WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way8Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim
    
    
        
class FactoryPhasedCache4Way16WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way16WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way16Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way32WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way32WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way32Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way64WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way64WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way64Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way128WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way128WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way128Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way256WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way256WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way256Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way512WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way512WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way512Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way1024WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way1024WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way1024Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        


class FactoryPhasedCache4Way2048WordsMemorySystem():
    
    build = current_sim.run['FactoryPhasedCache4Way2048WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4Way2048Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

#===============================================================================
# Phased DM Caches
#===============================================================================
class FactoryPhasedDMCache32WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache32WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory32WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache64WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache64WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory64WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache128WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache128WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory128WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache256WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache256WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory256WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache512WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache512WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory512WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache1024WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache1024WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory1024WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache2048WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache2048WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory2048WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

class FactoryPhasedDMCache4096WordsBaselineMemorySystem():
    
    build = current_sim.run['FactoryPhasedDMCache4096WordsBaselineMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactory4096WordsPhasedCacheBaseline.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = PhasedDMCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    
        

#===============================================================================
# Way Halting MS
#===============================================================================
class FactoryWayHalting4Way16WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way16WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way16Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryWayHalting4Way32WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way32WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way32Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryWayHalting4Way64WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way64WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way64Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryWayHalting4Way128WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way128WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way128Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryWayHalting4Way256WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way256WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way256Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryWayHalting4Way512WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way512WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way512Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


class FactoryWayHalting4Way1024WordsMemorySystem():
    
    build = current_sim.run['FactoryWayHalting4Way1024WordsMemorySystem']

    @staticmethod
    def Build():
        
        cache = CacheFactoryWayHalting4Way1024Words.Build()
        
        flash_memory = FlashMemory(MAIN_MEMORY_SIZE, FLASH_ENERGY_PER_READ)
        memory_system = WayHaltingCacheMemorySystem(cache, flash_memory)
        
        sim = Simulation(memory_system, trace_file)
        return sim    


