#include "cuddObj.hh"
#include <math.h>
#include <iostream>
#include <fstream>
#include <cassert>
#include <sstream>
#include <bitset>
#include <map>
#include <stdio.h>
#include <limits>


using namespace std;

int const NUMBER_OF_ADDRESS_BITS = 18;
int const NUMBER_OF_INDEX_BITS = 10;
int const MAX_NUMBER_OF_INDEX_BITS = 10;

// New type definition
typedef std::map<int, int> MemoryDict;
typedef std::map<std::pair<int, int>, int> PairDict;
typedef MemoryDict::const_iterator It;

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

#ifndef lint
static char rcsid[] UNUSED = "$Id: testobj.cc,v 1.7 2012/02/05 01:06:40 fabio Exp fabio $";
#endif

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/

static void testAdd(Cudd& mgr, int verbosity);


/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis    [Main program for testobj.]

  Description []

  SideEffects [None]

  SeeAlso     []

******************************************************************************/
int
main(int argc, char **argv)
{
    int verbosity = 0;

    if (argc == 2) {
        int retval = sscanf(argv[1], "%d", &verbosity);
        if (retval != 1)
            return 1;
    } else if (argc != 1) {
        return 1;
    }

    Cudd mgr(0,2);
    cout << "wataheu\n";
    testAdd(mgr,6);
    return 0;

} // main


int ConvertStrToInt(string data){
    int x;
    std::stringstream str(data);
    str >> x;
    return x;

}

MemoryDict GetLastMemoryAccessedCache(vector<int> program_trace){

    MemoryDict last_memory_access;

    int number_of_mem_accesses = program_trace.size();
    int address_i;

    for (int i = 0; i < number_of_mem_accesses; i++){
        address_i = program_trace[i];
        last_memory_access[address_i] = i;
    }

    return last_memory_access;

}


bitset<NUMBER_OF_ADDRESS_BITS> CalculateDCP(int address_i, int address_j){
//    cout << "\nxoring " << address_i << " , " << address_j << "\n";
    bitset<NUMBER_OF_ADDRESS_BITS> address_i_bits(address_i);
    bitset<NUMBER_OF_ADDRESS_BITS> address_j_bits(address_j);
    bitset<NUMBER_OF_ADDRESS_BITS> xor_addresses = address_i_bits ^ address_j_bits;
//    cout << xor_addresses;
    return xor_addresses;

}

void CalculateCP(bitset<NUMBER_OF_ADDRESS_BITS> DCP, vector<bitset<NUMBER_OF_ADDRESS_BITS> > &CP){

    // if the vector is empty, just add it the DCP
/*    cout << "\nCP ADDRESS " << &CP;
    cout << "\nSize " << CP.size();*/
    int size = CP.size();
    if (CP.empty()){
        CP.push_back(DCP);
        return;
    }
    bitset<NUMBER_OF_ADDRESS_BITS> temp;

    for (int i = 0; i < size; i++){
        // make a AND with all elements from CP, one at a time
//        cout << "\n\nDCP " << DCP.to_ulong() << " CP[" << i << "] " << CP[i].to_ulong() << "\n";
        temp = DCP & CP[i];
//        cout << "temp " << temp.to_ulong() << "\n";
        if (temp == DCP){
            CP[i] = temp;
            return;
        }
        else if (temp == CP[i])
            return;
        else{
            CP.push_back(DCP);
            return;
        }
    }

    return;

}


vector<bitset<NUMBER_OF_ADDRESS_BITS> > GeneratePatterns(int number_of_index_bits){
  
  cout << " GeneratePatterns \n"; 
  int number_of_addresses = pow(2, NUMBER_OF_ADDRESS_BITS);
  
  cout << number_of_addresses << "\n";
  
  vector<bitset<NUMBER_OF_ADDRESS_BITS> > patterns;
  
  int count_number_of_ones = 0;
  for (int i = 0; i < number_of_addresses; i++){
      bitset<NUMBER_OF_ADDRESS_BITS> temp(i);
      
      count_number_of_ones = 0;
      for (int j = 0; j < NUMBER_OF_ADDRESS_BITS; j++){
          if (temp[j] == 1)
              count_number_of_ones++;
      }

      if (count_number_of_ones == number_of_index_bits){
          patterns.push_back(temp);
      }
  }
  
  return patterns;
}


void Evaluate(vector<bitset<NUMBER_OF_ADDRESS_BITS> > &all_patterns, int number_of_index_bits, ADD &add_result){
    
    cout << "\nStarting evaluation of cache size with " << number_of_index_bits << " indexes \n";

    int minimum, temp_value;
    minimum = std::numeric_limits<int>::max();
      
    ADD eval_add;
    
    DdNode* n;
    DdNode min;
    int inputs[NUMBER_OF_ADDRESS_BITS];
    int bit;
    
    for (int p = 0; p < all_patterns.size(); p++){
        
        cout << "\nEvaluating: ";
        for (int s = 0; s < NUMBER_OF_ADDRESS_BITS; s++){
            bit =  all_patterns[p][s];
            cout << bit;
            inputs[s] = bit;
        }
      
      
        cout << "\n";
        eval_add = add_result.Eval(inputs);
        eval_add.print(2, 6);   
        
        n = eval_add.getNode();
        temp_value = n->type.value;
        if (temp_value < minimum)
            minimum = temp_value;
            min = *n;
        
    }

    // finding the minimum
    cout << "\nMinimum value: "; 
    cout <<  minimum;
    cout << "\n";
    cout << n;
    cout << "\n";  
  
    return;
  
}


static void
testAdd(
  Cudd& mgr,
  int verbosity)
{

    // initializing the ADD with background constant as plusInfinity, so we can
    // actually see the 0
    ADD pInf = mgr.plusInfinity();
    mgr.SetBackground(pInf);

    // creating the initial ADD vars and result
    ADD result = mgr.constant(0);

    vector<BDD> y(NUMBER_OF_ADDRESS_BITS);

    // initialize the ADD
    int i;
    for (i = 0; i < NUMBER_OF_ADDRESS_BITS; i++) {
        y[i] = mgr.bddVar(i);
    }

    bitset<NUMBER_OF_ADDRESS_BITS> address_i_bits;
    bitset<NUMBER_OF_ADDRESS_BITS> address_j_bits;
    bitset<NUMBER_OF_ADDRESS_BITS> DCP;

    // read from file and put it on an list of integers
    std::string line;
    int number;
    ifstream myfile ("tt.txt");

    int myints[] = {0, 1, 5, 6, 1, 5, 6, 7, 6};
    vector<int> program_trace; // (myints, myints + sizeof(myints) / sizeof(int) );
    
    // read the trace file and puts in the variable program_trace
    if (myfile.is_open())
    {
      do {
        getline (myfile,line);
        number = ConvertStrToInt(line);
        program_trace.push_back(number);
      }
      while ( myfile.good() );
      myfile.close();
    }


    // cache the last index of each address in a dict like structure
    // this is used to speed up the process of checking whether the current address
    // being analysed is the last reference being made to it,
    // if it is, then nothing should be done


    MemoryDict last_memory_access;
    last_memory_access = GetLastMemoryAccessedCache(program_trace);

    // main body of the algorithm
    // double loop to go over the addresses
    bool index_found;
    int number_of_mem_accesses = program_trace.size();
    int address_i, address_j;
    int last_access_of_address_i;

    pair<int, int> pair_of_addresses;
    
    BDD minterm, temp;

    int pair_found;

    for (int i = 0; i < number_of_mem_accesses; i++){

        PairDict edge_cache;
        vector<bitset<NUMBER_OF_ADDRESS_BITS> > CP;
        vector<BDD> CP_BDD;

        // used to cache pairs in the same edge (address_i -> address_j)

        address_i = program_trace[i];
        last_access_of_address_i = last_memory_access[address_i];

        index_found = last_access_of_address_i > i;

        // this means that there will be a reference to address_i in the future
        if (index_found){

            for (int j = i + 1; j < number_of_mem_accesses; j++){
                address_j = program_trace[j];

                // if the addresses match, we found an edge, that is, a list
                // of addresses for which the conflicts should be analysed
                if (address_i == address_j){

                    // add CP to ADD
                    temp = mgr.bddZero();

                    for (int each_cp = 0; each_cp < CP.size(); each_cp++){

                        minterm = mgr.bddOne();
                        for (int bit_index = 0; bit_index < NUMBER_OF_ADDRESS_BITS; bit_index++){
                            if (CP[each_cp][bit_index] == 1)
                                minterm *= ~y[bit_index];
                        }
                        temp += minterm;

                    }
                    result += temp.Add();

                    break;
                }


                // i will cache the expression (ADD) for each pair of addresses
                // because the same pair generates the same boolean expression
                // which when "ORed" with same pair, will be simplified
                // therefore there is no need to add it to the ADD
                if (address_i > address_j)
                    pair_of_addresses = make_pair(address_j, address_i);
                else
                    pair_of_addresses = make_pair(address_i, address_j);

                pair_found = edge_cache.count(pair_of_addresses);

                // if pair not found, then add expression to the ADD and add it to
                // the cache, otherwise, do nothing =)
                if (!pair_found){
                    edge_cache[pair_of_addresses] = 1;
                    
                    DCP = CalculateDCP(address_i, address_j);
                    CalculateCP(DCP, CP);
                }
            }
        }
    }

    cout << "\nresult", result.print(2,verbosity);
    cout << "\n\n";
    
    for (int n = NUMBER_OF_INDEX_BITS; n <= MAX_NUMBER_OF_INDEX_BITS; n++){
		vector<bitset<NUMBER_OF_ADDRESS_BITS> > patterns = GeneratePatterns(n);
		Evaluate(patterns, n, result);

    }


//    vector<bitset<NUMBER_OF_ADDRESS_BITS> > all_patterns_2k = GeneratePatterns(NUMBER_OF_INDEX_BITS + 1);
//    vector<bitset<NUMBER_OF_ADDRESS_BITS> > all_patterns_4k = GeneratePatterns(NUMBER_OF_INDEX_BITS + 2);
//
//    Evaluate(all_patterns_2k, NUMBER_OF_INDEX_BITS + 1, result);
//    Evaluate(all_patterns_4k, NUMBER_OF_INDEX_BITS + 2, result);
    
} // testAdd





