/******************************************************************************
* (C) Copyright 2011 <Company Name> All Rights Reserved
*
* MODULE:    DMCacheController
* DEVICE:     
* PROJECT:   
* AUTHOR:    vica   
* DATE:      2013 3:15:43 PM
*
* ABSTRACT:  You can customize the file content form Templates "verilog File"
*            
*******************************************************************************/

//Core
//ck
//arst
//
//AHB-Lite Bus
//ahbHSel      
//ahbHClk
//ahbHWrite    
//[1:0 ]ahbHSize     
//[1:0] ahbHTrans
//[NB_HADDR-1:0] ahbHAddr        
//[NB_HDATA-1:0] ahbHWData     
//ahbHReadyIn
//[NB_HDATA-1:0] ahbHRData     
//ahbHReadyOut   
//ahbHResp
//     
//Cache control signals
//Hit
//
//SRAM interface 
//[NB_CACHE_ADDR_BITS-1:0]Addr
//DataIn
//DataOut
//CE
//WE
//Register
//indexingConfig
`timescale 1 ns / 1 ps  

module DMCacheController #(
  // -- Module parameters
  parameter NB_CACHE_ADDR_BITS = 8,
  parameter NB_CACHE_HDATA     = 32,
  parameter NB_WORDS_PER_LINE  = 1,
  parameter NB_TAG             = 8,
  parameter NB_HADDR           = 16,
  parameter NB_HDATA           = 32
  )
  (
  // -- Inputs and Outputs
  // Core
//  input logic ck,
//  input logic arst,
  
  // AHB-Lite Bus
  input logic ahbHsel,
  input logic ahbResetN,
  input logic ahbHClk,
  input logic ahbHWrite,
  input logic [1:0] ahbHSize,
  input logic [1:0] ahbHTrans,  
  input logic [NB_HADDR-1:0] ahbHAddr,
  input logic [NB_HDATA-1:0] ahbHWData,
  input logic ahbHReadyIn,
  
  output logic [NB_HDATA-1:0] ahbHRData,     
  output logic ahbHReadyOut,   
  output logic ahbHResp,
  
  // Cache control signals
  output logic hit,
  
  // SRAM interface 
  output logic [NB_CACHE_ADDR_BITS-1:0] SRAMAddrRead,
  output logic [NB_CACHE_ADDR_BITS-1:0] SRAMAddrWrite,
  
  input logic [NB_TAG-1:0] SRAMTagInRead,
  input logic [NB_CACHE_HDATA-1:0] SRAMDataInRead,
  input logic SRAMValidBitInRead,
  
  
  output logic [NB_TAG-1:0] SRAMTagOutWrite,
  output logic [NB_CACHE_HDATA-1:0] SRAMDataOutWrite,
  output logic SRAMValidBitOutWrite,

  output logic SRAMreTagRead,
  output logic SRAMreDataRead,
  output logic SRAMreValidBitRead,
  
  output logic SRAMweTagWrite,
  output logic SRAMweDataWrite,
  output logic SRAMweValidBitWrite,
  
  output logic SRAMceTagRead,
  output logic SRAMceTagWrite,
  output logic SRAMceDataRead,
  output logic SRAMceDataWrite,
  output logic SRAMceValidBitRead,
  output logic SRAMceValidBitWrite,
  
  
  // NVM interface
  input logic NVMDataReady,
  input logic [NB_HDATA-1:0] NVMDataIn,
  output logic [NB_HADDR-1:0] NVMAddr,
  output logic NVMce,
  
    
  //  Register
  // it is -3 because the first 2 bits of the address are for byte offset, hence not used
  // in the mapping
  input logic [NB_HADDR-3:0] indexingConfig
  );
  
  // local declarations
  // signal provided by the AHB bus, basically saying if it is a read operation
  logic readEnabled, regReadEnabled;
  
  // signal from the DMCache core with info regarding the state of the reading operation
  // this is useful to generate waiting states on the ahb transfer in case there is a delay
  // when reading from the SRAM or NVM
  logic dataReady;
  
  logic regHit;
  
  // a registered version of the ahb address
  logic [NB_HADDR-1:0] regAddress;
  
  // the data read from either the NVM or the SRAM
  logic  [NB_HDATA-1:0] dataRead;
  
  // basic output functionality 
  // if there is a hit, the mux directs the data to come from the SRAM, otherwise
  // it will come from the NVM
  // this data read signal is connected to the AHB Slave which will forward this out 
  // via the bus
  always_comb begin
    dataRead = hit ? SRAMDataInRead : NVMDataIn;
    dataReady = regHit | NVMDataReady;
  end


//  Instantiations

  // AHB-Lite Slave 
  DMCacheControllerAHBSlave #(
    .NB_HDATA (NB_HDATA),
    .NB_HADDR (NB_HADDR)
  ) 
  u_DMCacheControllerAHBSlave (
    
    // -- Inputs
    .ahbHsel (ahbHsel),
    .ahbHClk (ahbHClk),
    .ahbResetN (ahbResetN),                  
    .ahbHWrite (ahbHWrite),
    .ahbHSize (ahbHSize),           
    .ahbHTrans (ahbHTrans),          
    .ahbHAddr (ahbHAddr),  
	  .ahbHWData(ahbHWData), 
    .ahbHReadyIn(ahbHReadyIn),
    
    .dataRead(dataRead),
    .dataReady(dataReady),
    
    // -- Outputs
    .ahbHRData(ahbHRData),
    .ahbHReadyOut(ahbHReadyOut),
    .ahbHResp(ahbHResp),
    
    .readEnabled(readEnabled),
    .regAddress(regAddress),
    .regReadEnabled (regReadEnabled)
  );
  
  //  Core
  
  DMCacheControllerCore #(
   .NB_HADDR (NB_HADDR),
   .NB_HDATA (NB_HDATA),
   .NB_CACHE_ADDR_BITS (NB_CACHE_ADDR_BITS),
   .NB_CACHE_HDATA (NB_CACHE_HDATA),
   .NB_WORDS_PER_LINE (NB_WORDS_PER_LINE),
   .NB_TAG (NB_TAG)
  )
  
  u_DMCacheControllerCore (
  
    .ck (ahbHClk),
//    .arst (arst),
    // inputs
    .readEnabled(readEnabled),
    .regReadEnabled (regReadEnabled),
    .ahbHAddr(ahbHAddr),
    .regAddress(regAddress),
    
    .SRAMTagInRead (SRAMTagInRead),
//    .SRAMDataInRead (SRAMDataInRead),
    .SRAMValidBitInRead (SRAMValidBitInRead),
    
    .NVMDataIn (NVMDataIn),
    .NVMce (NVMce),
    .NVMAddr (NVMAddr),
    
    .indexingConfig (indexingConfig),
    
    // outputs
    .SRAMAddrRead (SRAMAddrRead),
    .SRAMAddrWrite (SRAMAddrWrite),
    
    .SRAMTagOutWrite (SRAMTagOutWrite),
    .SRAMDataOutWrite (SRAMDataOutWrite),
    .SRAMValidBitOutWrite (SRAMValidBitOutWrite),
    
    .SRAMreTagRead (SRAMreTagRead),
    .SRAMreDataRead (SRAMreDataRead),
    .SRAMreValidBitRead (SRAMreValidBitRead),
    
    .SRAMweTagWrite (SRAMweTagWrite),
    .SRAMweDataWrite (SRAMweDataWrite),
    .SRAMweValidBitWrite (SRAMweValidBitWrite),
    
    .SRAMceTagRead (SRAMceTagRead),
    .SRAMceTagWrite (SRAMceTagWrite),
    
    .SRAMceDataRead (SRAMceDataRead),
    .SRAMceDataWrite (SRAMceDataWrite),
    
    .SRAMceValidBitRead (SRAMceValidBitRead),
    .SRAMceValidBitWrite (SRAMceValidBitWrite),
    
    .hit(hit),
    .regHit (regHit)
    
  );

endmodule
                            
/******************************************************************************
*             
*                 REVISION HISTORY:
*    
*******************************************************************************/
