/******************************************************************************
* (C) Copyright 2011 <Company Name> All Rights Reserved
*
* MODULE:    DMCacheControllerAHBSlave
* DEVICE:     
* PROJECT:   
* AUTHOR:    vica   
* DATE:      2013 3:18:45 PM
*
* ABSTRACT:  You can customize the file content form Templates "verilog File"
*            
*******************************************************************************/

module DMCacheControllerAHBSlave #(
  parameter NB_HADDR           = 16,
  parameter NB_HDATA           = 32
  )
  (
  input logic ahbHsel,
  input logic ahbHClk,
  input logic ahbResetN,
  input logic ahbHWrite,
  input logic [1:0] ahbHSize,   // NOTE: Hsize is ignored
  input logic [1:0] ahbHTrans,  
  input logic [NB_HADDR-1:0] ahbHAddr,        
  input logic [NB_HDATA-1:0] ahbHWData,     
  input logic ahbHReadyIn,
  input logic [NB_HDATA-1:0] dataRead,
  input logic dataReady,
  
  output logic [NB_HDATA-1:0] ahbHRData,     
  output logic ahbHReadyOut,   
  output logic ahbHResp,
  // registered version of ahbHAddr
  // since the transfers are pipelined, we need to keep the previous
  // address for one clock cycle
  output logic [NB_HADDR-1:0] regAddress,
  // basically tells whether a valid read transfer is being processed.
  output logic readEnabled,
  // and this is the registered counter part
  output logic regReadEnabled
  );
	
// HTRANS transfer type signal encoding
  `define TRN_IDLE   2'b00
  `define TRN_BUSY   2'b01
  `define TRN_NONSEQ 2'b10
  `define TRN_SEQ    2'b11
 
//// HRESP transfer response signal encoding
//  `define RSP_OKAY  1'b0
//  `define RSP_ERROR 2'b01
//  `define RSP_RETRY 2'b10
//  `define RSP_SPLIT 2'b11
  
  // local signals
  logic readOpEnabled;
//  logic [NB_HADDR-1:0] regAddress;['
//  logic readEnabled;
  
  
  always_comb begin
    // read is enabled when this slave is selected on Hsel, when write is 0, when ready in is 1 and 
    // when the transfer type is NONSEQ (the only one implemented)
//    $display("ahbHTrans %b", ahbHTrans);
//    $display("ahbHsel %b", ahbHsel);
//    $display("ahbHWrite %b", ahbHWrite);
//    $display("ahbHReadyIn %b", ahbHReadyIn);
    readOpEnabled = ahbHsel & !ahbHWrite & ahbHReadyIn & (ahbHTrans == `TRN_NONSEQ);
//    $display("readOpEnabled %b\n\n", readOpEnabled);
  end
  
  always_comb begin
    readEnabled = readOpEnabled;
  end


  // registered versions of ahb address and read enabled signals
  always_ff @ (posedge ahbHClk or negedge ahbResetN)  begin
  
    if (!ahbResetN)
      begin
        regAddress <= {NB_HADDR {1'b0}};
        regReadEnabled <= 1'b0;
      end
    else
      begin
//        if (readOpEnabled)
//          begin
            regAddress <= ahbHAddr;
            regReadEnabled <= readEnabled; //1'b1;
//          end
      end
  end
  
  //HRData block
  // the output data is the same as the one coming from the DMCacheController
  always_comb begin
    ahbHRData = dataRead;
  end  
  
  // HResp
  // according to the NVM controller, this signal is always okay when reading
  always_comb begin
    ahbHResp <= 1'b0;
  end
  
  // HReadyOut block 
  // asserted to 1 during reset (in accordance with the specification)
  // otherwise it depends on dataReady, which is a signal from the DMCacheController
  // with the status from the memories
  always_ff @ (posedge ahbHClk or negedge ahbResetN)  begin
    if (!ahbResetN)
      ahbHReadyOut <= 1'b1;
    else
      ahbHReadyOut <= dataReady;      
  end
  
	
endmodule

/******************************************************************************
*
* REVISION HISTORY:
*    
*******************************************************************************/
