`timescale 1 ns / 1 ps
/******************************************************************************
* (C) Copyright 2011 <Company Name> All Rights Reserved
*
* MODULE:    DMCacheControllerCore
* DEVICE:     
* PROJECT:   
* AUTHOR:    vica   
* DATE:      2013 3:16:26 PM
*
* ABSTRACT:  You can customize the file content form Templates "verilog File"
*            
*******************************************************************************/

module DMCacheControllerCore #(
  // -- Module parameters
  parameter NB_HADDR           = 16,
  parameter NB_HDATA           = 32,
  parameter NB_CACHE_ADDR_BITS = 8,
  parameter NB_CACHE_HDATA     = 32,
  parameter NB_WORDS_PER_LINE  = 1,
  parameter NB_TAG             = 8
  )
  (
  // -- Inputs and Outputs
  // Core
  input logic ck,
  input logic arst,
  
  // from AHB
  input logic [NB_HADDR-1:0] regAddress,
  input logic readEnabled,
  input logic regReadEnabled,
  input logic [NB_HADDR-1:0] ahbHAddr,
  
  // Cache control signals
  output logic hit,
  output logic regHit,
  
  // SRAM interface 
  output logic [NB_CACHE_ADDR_BITS-1:0] SRAMAddrRead,
  output logic [NB_CACHE_ADDR_BITS-1:0] SRAMAddrWrite,
  
  input logic [NB_TAG-1:0] SRAMTagInRead,
//  input logic [NB_CACHE_HDATA-1:0] SRAMDataInRead,
  input logic SRAMValidBitInRead,
  
  
  output logic [NB_TAG-1:0] SRAMTagOutWrite,
  output logic [NB_CACHE_HDATA-1:0] SRAMDataOutWrite,
  output logic SRAMValidBitOutWrite,

  output logic SRAMreTagRead,
  output logic SRAMreDataRead,
  output logic SRAMreValidBitRead,
  
  output logic SRAMweTagWrite,
  output logic SRAMweDataWrite,
  output logic SRAMweValidBitWrite,
  
  output logic SRAMceTagRead,
  output logic SRAMceTagWrite,
  output logic SRAMceDataRead,
  output logic SRAMceDataWrite,
  output logic SRAMceValidBitRead,
  output logic SRAMceValidBitWrite,
  
  
  // NVM interface
  input logic [NB_HDATA-1:0] NVMDataIn,
  output logic [NB_HADDR-1:0] NVMAddr,
  output logic NVMce,
  
    
  //  Register
  // it is -3 because the first 2 bits of the address are for byte offset, hence not used
  // in the mapping
  input logic [NB_HADDR-3:0] indexingConfig
  
  );
  
  // holds the address coming directly from the AHB bus
  // converted to the cache address (mapping)
  logic [NB_CACHE_ADDR_BITS-1:0] cacheAHBAddress;
  
  // holds the registered address mapped to the cache address
  logic [NB_CACHE_ADDR_BITS-1:0] cacheRegAddress;
  
  // the tag portion of the address
  logic [NB_TAG-1:0] ahbTagAddress;
  logic [NB_TAG-1:0] regTagAddress;
  
  
  logic hitW, hitR;
  
  // a registered version of the hit signal, needed on the data phase when writing 
  // back the data into the cache
//  logic regHit;
  
  // a registered version of the read enabled signal.
  // this is used to know that a write should be performed in the next cycle, after a read
  // if a hit did not occur
//  logic regReadEnabled;


  assign ahbTagAddress = fu_map(ahbHAddr, indexingConfig);
  assign regTagAddress = fu_map(regAddress, indexingConfig);
  
  // this gets the tag bits from the full address
  // this is needed because the tag bits change according to the indexingConfig
  function logic[NB_TAG-1:0] fu_map (logic [NB_HADDR-1:0] addressIn, logic [NB_HADDR-3:0] indexingConfig);
  
    logic [NB_HADDR-1:0] temp;
    integer i;  
    integer index;
    
    index = 0;
  
    // perform a mask on the address with the indexing configuration
//    $display("addressIn %h", addressIn);
//    $display("indexingConfig %b", indexingConfig);
    temp = addressIn & {~indexingConfig, 2'b00};
//    $display("TEMP %b\n\n", temp);
    for (i = 2; i < NB_HADDR; i++)
      begin
        if (indexingConfig[i-2] == 1'b0)
          begin
            fu_map[index] = temp[i];
            index = index + 1;
          end
      end
  endfunction

  
  
  // NVM signals
  always_comb begin
    NVMAddr = regAddress;
    NVMce = ~regHit;
  end
  
  
  // tag 
//  always_comb begin
//    ahbTagAddress = ahbHAddr[NB_HADDR-1:NB_HADDR-NB_TAG]; 
//    regTagAddress = regAddress[NB_HADDR-1:NB_HADDR-NB_TAG]; 
//  end

  // read enabled and chip enabled for tag memory
  // TODO: for now, they are always active (asynch read, still don't know when I can 
  // de-activate it) 
  // Chip Enable is active 1, Read is active 0
  always_comb begin
    SRAMceTagRead = 1'b1 & readEnabled;
    SRAMreTagRead = 1'b0 & ~readEnabled;
    
    SRAMceValidBitRead = 1'b1 & readEnabled;
    SRAMreValidBitRead = 1'b0 & ~readEnabled;
  end
  
  
  // when reading from the cache, always use the current address on the ahb bus
  // because the reading happens on the address phase
  // however when writing, the address comes from the registered version of the ahb address
  // because writing happens in the data phase of the previous address phase
  always_comb begin
    SRAMAddrRead = cacheAHBAddress;
//    $display("SRAMAddrRead %h\n", SRAMAddrRead);
    SRAMAddrWrite = cacheRegAddress;
  end
  
  // hit signal
  always @(*) begin //SRAMTagInRead or SRAMValidBitInRead or ahbTagAddress or SRAMAddrWrite or SRAMTagOutWrite or SRAMAddrRead or SRAMweDataWrite) begin
    if (SRAMAddrRead == SRAMAddrWrite)
      if (SRAMweDataWrite)
        hit = (SRAMTagOutWrite == ahbTagAddress);
      else
        hit = ((SRAMTagInRead == ahbTagAddress) & SRAMValidBitInRead);
//      hit = (SRAMTagOutWrite == ahbTagAddress) & SRAMweDataWrite;
    else
      hit = ((SRAMTagInRead == ahbTagAddress) & SRAMValidBitInRead);
  
//    hitW = ((SRAMTagOutWrite == ahbTagAddress) & (SRAMAddrRead == SRAMAddrWrite) & SRAMweDataWrite);
//    hitR = ((SRAMTagInRead == ahbTagAddress) & SRAMValidBitInRead);
//    hit = hitR | hitW;
//    $display("SRAMTagInRead %h", SRAMTagInRead);
//    $display("ahbTagAddress %h", ahbTagAddress);
//    $display("SRAMValidBitInRead %h", SRAMValidBitInRead);
    
  end
  
  always_ff @ (posedge ck) begin
    regHit <= hit;
//    regReadEnabled <= readEnabled;
  end
  
  
  // read enable and chip enable for data memory
  always_comb begin
    SRAMreDataRead = ~hit & ~readEnabled;
    SRAMceDataRead = hit & readEnabled;
  end
  

  // write enable and chip enable for writing data and tag
  always_comb begin
    SRAMweDataWrite = ~regHit & regReadEnabled;
    SRAMceDataWrite = ~regHit & regReadEnabled;
    
    SRAMweTagWrite = ~regHit & regReadEnabled;
    SRAMceTagWrite = ~regHit & regReadEnabled;
    
    SRAMweValidBitWrite = ~regHit & regReadEnabled;
    SRAMceValidBitWrite = ~regHit & regReadEnabled;
  end
  
  // tag and data to be written to the SRAM
  always_comb begin
    SRAMTagOutWrite = regTagAddress;
    SRAMDataOutWrite = NVMDataIn;
    SRAMValidBitOutWrite = 1'b1;
  end
  
  // Instantiations
  DMCacheControllerAddressMapping #(
    .NB_HADDR (NB_HADDR),
    .NB_CACHE_ADDR_BITS (NB_CACHE_ADDR_BITS)
  )
  uAHB_DMCacheControllerAddressMapping (
    .addressIn (ahbHAddr),
    .indexingConfig (indexingConfig),
    .addressOut (cacheAHBAddress)
  );
	
  DMCacheControllerAddressMapping #(
    .NB_HADDR (NB_HADDR),
    .NB_CACHE_ADDR_BITS (NB_CACHE_ADDR_BITS)
  )
  uReg_DMCacheControllerAddressMapping (
    .addressIn (regAddress),
    .indexingConfig (indexingConfig),
    .addressOut (cacheRegAddress)
  );
	
endmodule

/******************************************************************************
*
* REVISION HISTORY:
*    
*******************************************************************************/
