module DMCacheMemory #(
  parameter NB_ADDRESS = 5,
  parameter NB_DATA    = 32,
  parameter NB_TAG     = 11,
  parameter MEM_DEPTH  = 1 << NB_ADDRESS 
  )
  (
  input clk,
  input arst,
  
  input logic [NB_ADDRESS-1:0] addrRead,
  input logic [NB_ADDRESS-1:0] addrWrite,
  
  output logic [NB_TAG-1:0] tagOutRead,
  output logic [NB_DATA-1:0] dataOutRead,
  output logic validBitOutRead,
  
  
  input logic [NB_TAG-1:0] tagInWrite,
  input logic [NB_DATA-1:0] dataInWrite,
  input logic validBitInWrite,

  input logic reTag,
  input logic reData,
  input logic reValidBit,
  
  input logic weTag,
  input logic weData,
  input logic weValidBit,
  
  input logic ceTagRead,
  input logic ceTagWrite,
  input logic ceDataRead,
  input logic ceDataWrite,
  input logic ceValidBitRead,
  input logic ceValidBitWrite

  );
  
    
  // instantiations
  // Valid Bit Array
  OneBitMemory #(
    .NB_ADDRESS (NB_ADDRESS),
    .MEM_DEPTH (MEM_DEPTH)
  )
  u_ValidBit (
    .clk (clk),
    .arst (arst),
    // read port
    .addrRead (addrRead),
    .ceRead (ceValidBitRead),
    .re (reValidBit),
    .dataOutRead (validBitOutRead),
    
    // write port
    .addrWrite (addrWrite),
    .ceWrite (ceValidBitWrite),
    .we (weValidBit),
    .dataInWrite (validBitInWrite)
  );
    
  // Tag Array
  TagMemory #(
    .NB_ADDRESS (NB_ADDRESS),
    .NB_DATA (NB_TAG)
//    .MEM_DEPTH (MEM_DEPTH)
  )
  u_TagArray(
    .clk (clk),

    // read port
    .addrRead (addrRead),
    .ceRead (ceTagRead),
    .re (reTag),
    .dataOutRead (tagOutRead),
    
    // write port
    .addrWrite (addrWrite),
    .ceWrite (ceTagWrite),
    .we (weTag),
    .dataInWrite (tagInWrite)
  );
  
  // Data Array
  DataMemory #(
    .NB_ADDRESS (NB_ADDRESS),
    .NB_DATA (NB_DATA)
  )
  u_DataArray(
    .clk (clk),
    
    // read port
    .addrRead (addrRead),
    .ceRead (ceDataRead),
    .re (reData),
    .dataOutRead (dataOutRead),
    
    // write port
    .addrWrite (addrWrite),
    .ceWrite (ceDataWrite),
    .we (weData),
    .dataInWrite (dataInWrite)
  );
    
endmodule 
