`timescale 1 ns / 1 ps  

module DMCacheSystem #(
  // -- Module parameters
  parameter NB_HADDR           = 18,
  parameter NB_CACHE_HDATA     = 32,
  parameter NB_CACHE_ADDR_BITS = 6,
  parameter NB_WORDS_PER_LINE  = 1,
  parameter NB_TAG             = NB_HADDR - NB_CACHE_ADDR_BITS - 2,
  parameter NB_HDATA           = 32
  )
  (
  // -- Inputs and Outputs
  // Core
//  input logic ck,
//  input logic arst,
  
  // AHB-Lite Bus
  input logic ahbHsel,
  input logic ahbResetN,
  input logic ahbHClk,
  input logic ahbHWrite,
  input logic [1:0] ahbHSize,
  input logic [1:0] ahbHTrans,  
  input logic [NB_HADDR-1:0] ahbHAddr,
  input logic [NB_HDATA-1:0] ahbHWData,
  input logic ahbHReadyIn,
  
  output logic [NB_HDATA-1:0] ahbHRData,     
  output logic ahbHReadyOut,   
  output logic ahbHResp,
  
  // Cache control signals
  output logic hit,
  
  // NVM interface
  input logic NVMDataReady,
  input logic [NB_HDATA-1:0] NVMDataIn,
  output logic [NB_HADDR-1:0] NVMAddr,
  output logic NVMce,
  
  //  Register
  // it is -3 because the first 2 bits of the address are for byte offset, hence not used
  // in the mapping
  input logic [NB_HADDR-3:0] indexingConfig
  );
  
  // SRAM
  logic [NB_CACHE_ADDR_BITS-1:0] SRAMAddrRead;
  logic [NB_CACHE_ADDR_BITS-1:0] SRAMAddrWrite;

  logic [NB_TAG-1:0] SRAMTagRead;
  logic [NB_CACHE_HDATA-1:0] SRAMDataRead;
  logic SRAMValidBitRead;

  logic [NB_TAG-1:0] SRAMTagWrite;
  logic [NB_CACHE_HDATA-1:0] SRAMDataWrite;
  logic SRAMValidBitWrite;

  logic SRAMreTagRead;
  logic SRAMreDataRead;
  logic SRAMreValidBitRead;

  logic SRAMweTagWrite;
  logic SRAMweDataWrite;
  logic SRAMweValidBitWrite; 

  logic SRAMceTagRead;
  logic SRAMceTagWrite;
  logic SRAMceDataRead;
  logic SRAMceDataWrite;
  logic SRAMceValidBitRead;
  logic SRAMceValidBitWrite;      
    
  
  
  // SRAM memory
  DMCacheMemory # (
    .NB_ADDRESS (NB_CACHE_ADDR_BITS),
    .NB_DATA (NB_CACHE_HDATA),
    .NB_TAG (NB_TAG)
  )
  u_DMCacheMemory (
    .arst (ahbResetN),
    .clk (ahbHClk),
    
    .addrRead (SRAMAddrRead),
    .addrWrite (SRAMAddrWrite),

    .tagOutRead (SRAMTagRead),
    .dataOutRead (SRAMDataRead),
    .validBitOutRead (SRAMValidBitRead),

    .tagInWrite (SRAMTagWrite),
    .dataInWrite (SRAMDataWrite),
    .validBitInWrite (SRAMValidBitWrite),

    .reTag (SRAMreTagRead),
    .reData (SRAMreDataRead),
    .reValidBit (SRAMreValidBitRead),

    .weTag (SRAMweTagWrite),
    .weData (SRAMweDataWrite),
    .weValidBit (SRAMweValidBitWrite),  

    .ceTagRead (SRAMceTagRead),
    .ceTagWrite (SRAMceTagWrite),
    .ceDataRead (SRAMceDataRead),
    .ceDataWrite (SRAMceDataWrite),
    .ceValidBitRead (SRAMceValidBitRead),
    .ceValidBitWrite (SRAMceValidBitWrite)
  );
  
  // DUT
  DMCacheController # (
    .NB_CACHE_ADDR_BITS (NB_CACHE_ADDR_BITS),
    .NB_CACHE_HDATA (NB_CACHE_HDATA),
    .NB_HADDR (NB_HADDR),
    .NB_HDATA (NB_HDATA),
    .NB_TAG (NB_TAG)
  ) u_DMCacheController (
    // AHB signals  
    // inputs
//    .ck (ahbHClk),
//    .arst (ahbResetN),
    
    .ahbHsel (ahbHsel),
    
    .ahbHClk (ahbHClk),
    .ahbHWrite (ahbHWrite),
    .ahbResetN (ahbResetN),
    .ahbHSize (ahbHSize),
    .ahbHTrans (ahbHTrans),
    .ahbHAddr (ahbHAddr),
    .ahbHWData (ahbHWData),
    .ahbHReadyIn (ahbHReadyIn),
    // outputs
    .ahbHRData (ahbHRData),
    .ahbHReadyOut (ahbHReadyOut),
    .ahbHResp (ahbHResp),
    
    
    // specific DM signals
    .hit (hit),
    .indexingConfig (indexingConfig),
  
    // RAM memory signals
    .SRAMAddrRead (SRAMAddrRead),
    .SRAMAddrWrite (SRAMAddrWrite),

    .SRAMTagInRead (SRAMTagRead),
    .SRAMDataInRead (SRAMDataRead),
    .SRAMValidBitInRead (SRAMValidBitRead),

    .SRAMTagOutWrite (SRAMTagWrite),
    .SRAMDataOutWrite (SRAMDataWrite),
    .SRAMValidBitOutWrite (SRAMValidBitWrite),

    .SRAMreTagRead (SRAMreTagRead),
    .SRAMreDataRead (SRAMreDataRead),
    .SRAMreValidBitRead (SRAMreValidBitRead),

    .SRAMweTagWrite (SRAMweTagWrite),
    .SRAMweDataWrite (SRAMweDataWrite),
    .SRAMweValidBitWrite (SRAMweValidBitWrite),  

    .SRAMceTagRead (SRAMceTagRead),
    .SRAMceTagWrite (SRAMceTagWrite),
    .SRAMceDataRead (SRAMceDataRead),
    .SRAMceDataWrite (SRAMceDataWrite),
    .SRAMceValidBitRead (SRAMceValidBitRead),
    .SRAMceValidBitWrite (SRAMceValidBitWrite),
    
    // NVM
    .NVMAddr (NVMAddr),
    .NVMce (NVMce),
    .NVMDataIn (NVMDataIn),
    .NVMDataReady (NVMDataReady)
  );
  
endmodule